
#ifndef _H_TEXT_UTIL_LOADED
#define _H_TEXT_UTIL_LOADED 1


/* text_util */



class  TextZeileType
{
   public:
    TextZeileType(char *zeile);
   ~TextZeileType();
   void TextFreeTextZeilenListe (  );
   
   char *string;
   class TextZeileType *nachfolger;

} ;


class TextType
{
   public:
   TextType();
   TextType(class TextType *original);
   TextType(char *string);
   ~TextType();

   int TextGetZeilenAnzahl ( );

   bool TextAddTextZeile ( char *zeile );

   void TextInsertTextZeile ( char *zeile ,int position );

   char *TextGetZeile (  int zeilen_nummer );

   bool TextReplaceZeile (  int zeilen_nummer , 
        char *neuer_string );

   void TextRemoveZeile ( int zeilennummer );

   void TextRemoveAlleZeilen ( );


   void TextPrint ( );


   bool TextSaveText ( char *dateiname );

   bool TextLoadText ( char *dateiname );

   static TextType *TextAllocTextAusDatei ( char *dateiname );


   private:
   void initFelder();
   int anzahl_zeilen;	   
   TextZeileType * wurzel;  
   TextZeileType * hinten;  
} ;




#endif /* _TEXT_UTIL_LOADED */

